# QueryDam
[![Build Status](https://travis-ci.org/romankuznietsov/query_dam.svg)](https://travis-ci.org/romankuznietsov/query_dam)

QueryDam is a tool for observing changes in ActiveRecord relation results.
It uses Redis to store changes metadata.

## Installation

Add it to your gemfile Gemfile:

    gem 'query_dam'

## Usage

First, include `QueryDam::Trackable` in every model you want to track.

~~~ruby
class Item < ActiveRecord::Base
  include QueryDam::Trackable
end
~~~

Then start watching updates of the relation:

~~~ruby
relation = Item.where(name: 'pencil')
tracking_key = QueryDam.watch_relation(relation)
~~~

To get updates, call `QueryDam.get_updates(tracking_key)`.
The result is a hash with 3 keys: 
* `:model` - the model of the relation,
* `:updates` - relation containing records that were updated or entered the scope of the relation
* `:exclusions` - array of ids of records that left the scope of the relation

The call to `get_updates` clears updates/exclusions.

~~~ruby
Item.create(name: 'pencil')
QueryDam.get_updates(tracking_key)
# =>
# {:model=>Item, 
#  :updates=>#<ActiveRecord::Relation [#<Item id: 1, name: "pencil">]>, 
#  :exclusions=>[]}

QueryDam.get_updates(tracking_key)
# =>
# {:model=>Item, 
#  :updates=>#<ActiveRecord::Relation []>, 
#  :exclusions=>[]}

Item.first.destroy

QueryDam.get_updates(tracking_key)
# =>
# {:model=>Item, 
#  :updates=>#<ActiveRecord::Relation []>, 
#  :exclusions=>[1]}
~~~

If updates are not requested for a certain amount of time (5 minutes by default),
QueryDam will stop observing changes. You can set up expiration time (in an initializer, for example).

~~~ruby
QueryDam.expiration = 5.minutes
~~~

## Contributing

1. Fork it ( https://github.com/[my-github-username]/query_dam/fork )
2. Create your feature branch (`git checkout -b my-new-feature`)
3. Commit your changes (`git commit -am 'Add some feature'`)
4. Push to the branch (`git push origin my-new-feature`)
5. Create a new Pull Request
